class User < ActiveRecord::Base
  
  hobo_user_model # Don't put anything above this
  
  fields do
    name :string, :unique
    email_address :email_address, :unique, :login => true
    administrator :boolean, :default => false
    timestamps
  end
  
  has_many :championships, :foreign_key => "championship_manager_id", :dependent => :destroy 
  has_many :games, :foreign_key => "game_manager_id", :dependent => :destroy
  has_many :championship_memberships, :dependent => :destroy
  has_many :game_memberships, :dependent => :destroy
  has_many :joined_championships, :through => :championship_memberships, :source => :championship
  has_many :joined_games, :through => :game_memberships, :source => :game
  
  # This gives admin rights to the first sign-up.
  # Just remove it if you don't want that
  before_create { |user| user.administrator = true if count == 0 }
  
  
  # --- Signup lifecycle --- #
  
  lifecycle do
    
    state :active
    state :inactive, :default => true
    
    create :signup, :available_to => "Guest",
    :params => [:name, :email_address, :password, :password_confirmation],
    :new_key => true, :become => :inactive do
      UserMailer.deliver_account_activation(self, lifecycle.generate_key) 
    end 
    
    transition :activate_account, { :inactive => :active }, :available_to => :key_holder
    
    # resend activation, issued by guest
    transition :request_password_reset,
    { :inactive => :inactive },
    :new_key => true do  
      UserMailer.deliver_activation(self, lifecycle.key) 
    end 
    
    # deliver forgotten password 
    transition :request_password_reset,
    { :active => :active },
    :available_to => 'Guest',
    :new_key => true do  
      UserMailer.deliver_forgot_password(self, lifecycle.key) 
    end 
    
    # do the actual reset
    transition :reset_password,
    { :active => :active },
    :available_to => :key_holder,
    :update => [ :password, :password_confirmation ] 
    
    
  end
  
  
  # --- Permissions --- #
  
  def create_permitted?
    false
  end
  
  def update_permitted?
    acting_user.administrator? || (acting_user == self && only_changed?(:crypted_password, :email_address))
    # Note: crypted_password has attr_protected so although it is permitted to change, it cannot be changed
    # directly from a form submission.
  end
  
  def destroy_permitted?
    acting_user.administrator?
  end
  
  def view_permitted?(field)
    true
  end
  
end
